"use strict";
/*!
 * Copyright 2020, OpenTelemetry Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const NoopMeterProvider_1 = require("../metrics/NoopMeterProvider");
/**
 * Singleton object which represents the entry point to the OpenTelemetry Metrics API
 */
class MetricsAPI {
    /** Empty private constructor prevents end users from constructing a new instance of the API */
    constructor() {
        this._meterProvider = NoopMeterProvider_1.NOOP_METER_PROVIDER;
    }
    /** Get the singleton instance of the Metrics API */
    static getInstance() {
        if (!this._instance) {
            this._instance = new MetricsAPI();
        }
        return this._instance;
    }
    /**
     * Set the current global meter. Returns the initialized global meter provider.
     */
    setGlobalMeterProvider(provider) {
        this._meterProvider = provider;
        return provider;
    }
    /**
     * Returns the global meter provider.
     */
    getMeterProvider() {
        return this._meterProvider;
    }
    /**
     * Returns a meter from the global meter provider.
     */
    getMeter(name, version) {
        return this.getMeterProvider().getMeter(name, version);
    }
}
exports.MetricsAPI = MetricsAPI;
//# sourceMappingURL=metrics.js.map